@@ -49,7 +49,7 @@ static inline void core_get_tls(CoreEntry *pcore, tls_t *ptls)
 		COPY_TLS(read_exec_only);
 		COPY_TLS(limit_in_pages);
 		COPY_TLS(seg_not_present);
-		COPY_TLS(useable);
+		COPY_TLS(usable);
 #undef COPY_TLS
 	}
 }
