@@ -1310,6 +1310,14 @@ int cr_service_work(int sk)
 	int ret = -1;
 	CriuReq *msg = 0;
 
+	/*
+	 * util_init initializes criu_run_id and compel_run_id so that sockets
+	 * are generated with an unique name identifying the specific process
+	 * even in cases where multiple processes with the same pid in
+	 * different pid namespaces are sharing the same network namespace.
+	 */
+	util_init();
+
 more:
 	opts.mode = CR_SWRK;
 
