@@ -21,7 +21,7 @@ enum faults {
 	FI_CORRUPT_EXTREGS = 134,
 	FI_DONT_USE_PAGEMAP_SCAN = 135,
 	FI_DUMP_CRASH = 136,
-	FI_DISABLE_FREEZE_CGROUP = 137,
+	FI_COMPEL_INTERRUPT_ONLY_MODE = 137,
 	FI_PLUGIN_CUDA_FORCE_ENABLE = 138,
 	FI_MAX,
 };
