@@ -2,6 +2,7 @@
 #define __CR_SEIZE_H__
 
 extern int collect_pstree(void);
+extern int checkpoint_devices(void);
 struct pstree_item;
 extern void pstree_switch_state(struct pstree_item *root_item, int st);
 extern const char *get_real_freezer_state(void);
@@ -9,6 +10,6 @@ extern bool alarm_timeouted(void);
 
 extern char *task_comm_info(pid_t pid, char *comm, size_t size);
 extern char *__task_comm_info(pid_t pid);
-extern void dont_use_freeze_cgroup(void);
+extern void set_compel_interrupt_only_mode(void);
 
 #endif
