@@ -1,46 +1,17 @@
 #!/bin/bash
 
-# shellcheck disable=SC1091,SC2015
-
 set -x -e -o pipefail
 
-./apt-install \
-    apt-transport-https \
-    ca-certificates \
-    curl \
-    software-properties-common
-
-curl -fsSL https://download.docker.com/linux/ubuntu/gpg | sudo apt-key add -
-
-add-apt-repository \
-   "deb [arch=amd64] https://download.docker.com/linux/ubuntu \
-   $(lsb_release -cs) \
-   stable test"
-
-./apt-install docker-ce
-
-. /etc/lsb-release
-
-# overlayfs with current Ubuntu kernel breaks CRIU
-# https://bugs.launchpad.net/ubuntu/+source/linux-azure/+bug/1967924
-# Use devicemapper storage drive as a work-around
-echo '{ "experimental": true, "storage-driver": "devicemapper" }' > /etc/docker/daemon.json
+# docker checkpoint and restore is an experimental feature
+echo '{ "experimental": true }' > /etc/docker/daemon.json
+service docker restart
 
 CRIU_LOG='/criu.log'
 mkdir -p /etc/criu
 echo "log-file=$CRIU_LOG" > /etc/criu/runc.conf
 
-service docker stop
-systemctl stop containerd.service
-
-# Always use the latest containerd release.
-# Restore with containerd versions after v1.2.14 and before v1.5.0-beta.0 are broken.
-# https://github.com/checkpoint-restore/criu/issues/1223
-CONTAINERD_DOWNLOAD_URL=$(curl -s https://api.github.com/repos/containerd/containerd/releases/latest | grep '"browser_download_url":.*/containerd-.*-linux-amd64.tar.gz.$' | cut -d\" -f4)
-wget -nv "$CONTAINERD_DOWNLOAD_URL" -O - | tar -xz -C /usr/
-
-systemctl restart containerd.service
-service docker restart
+# Test checkpoint/restore with action script
+echo "action-script /usr/bin/true" | sudo tee /etc/criu/default.conf
 
 export SKIP_CI_TEST=1
 
@@ -88,17 +59,35 @@ checkpoint_container () {
 	docker wait cr
 }
 
-restore_container () {
-	CHECKPOINT_NAME=$1
-
-	docker start --checkpoint "$CHECKPOINT_NAME" cr 2>&1 | tee log || {
+print_logs () {
 	cat "$(grep log 'log file:' | sed 's/log file:\s*//')" || true
 		docker logs cr || true
 		cat $CRIU_LOG || true
 		dmesg
 		docker ps
 		exit 1
-	}
+}
+
+declare -i max_restore_container_tries=3
+
+restore_container () {
+	CHECKPOINT_NAME=$1
+
+	for i in $(seq $max_restore_container_tries); do
+		docker start --checkpoint "$CHECKPOINT_NAME" cr 2>&1 | tee log && break
+
+		# FIXME: There is a race condition in docker/containerd that causes
+		# docker to occasionally fail when starting a container from a
+		# checkpoint immediately after the checkpoint has been created.
+		# https://github.com/moby/moby/issues/42900
+		if grep -Eq '^Error response from daemon: failed to upload checkpoint to containerd: commit failed: content sha256:.*: already exists$' log; then
+			echo "Retry container restore: $i/$max_restore_container_tries"
+			sleep 1;
+		else
+			print_logs
+		fi
+
+	done
 }
 
 # Scenario: Create multiple containers and checkpoint and restore them once
