@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 # Test criu errno
 
 import socket, os, errno
@@ -42,7 +42,7 @@ class test:
 
     def check_resp(self, resp, typ, err):
         if resp.type != typ:
-            raise Exception('Unexpected responce type ' + str(resp.type))
+            raise Exception('Unexpected response type ' + str(resp.type))
 
         if resp.success:
             raise Exception('Unexpected success = True')
@@ -67,6 +67,7 @@ class test:
         req = self.get_base_req()
         req.type = rpc.DUMP
         req.opts.pid = pid
+        req.opts.network_lock = rpc.SKIP
 
         self.send_req(req)
         resp = self.recv_resp()
@@ -84,6 +85,7 @@ class test:
         req = self.get_base_req()
         req.type = rpc.DUMP
         req.opts.leave_running = True
+        req.opts.network_lock = rpc.SKIP
 
         self.send_req(req)
         resp = self.recv_resp()
