@@ -3,6 +3,7 @@
 set -e
 
 CRIU=./criu
+FAIL=1
 
 export PROTODIR=`readlink -f "${PWD}/../../protobuf"`
 
@@ -19,6 +20,13 @@ function stop_server {
 	title_print "Shutdown service server"
 	kill -SIGTERM $(cat build/pidfile)
 	unlink build/pidfile
+	if [ "${FAIL}" == "1" ]; then
+		for i in build/output*; do
+			echo "File: $i"
+			cat $i
+		done
+		find . -name "*.log" -print -exec cat {} \; || true
+	fi
 }
 
 function test_c {
@@ -51,7 +59,7 @@ function test_restore_loop {
 	title_print "Dump loop process"
 	# So theoretically '-j' (--shell-job) should not be necessary, but on alpine
 	# this test fails without it.
-	${CRIU} dump -j -v4 -o dump-loop.log -D build/imgs_loop -t ${P}
+	${CRIU} dump -j -v4 -o dump-loop.log --network-lock skip -D build/imgs_loop -t ${P}
 
 	title_print "Run restore-loop"
 	./restore-loop.py build/criu_service.socket build/imgs_loop
@@ -80,6 +88,8 @@ test_restore_loop
 test_ps
 test_errno
 
+FAIL=0
+
 stop_server
 
 trap 'echo "Success"' EXIT
